require "import"
import "android.app.*"
import "android.content.*"
import "android.os.*"
import "android.view.*"
import "android.widget.*"
import "android.media.*"
import "java.io.File"
import "java.io.FileWriter"
import "java.io.BufferedWriter"
import "java.util.ArrayList"
import "java.util.Scanner"

-- Constants
local STORAGE_PATH = "/storage/emulated/0/解说/Tools/radio-player/data.txt"

-- UI Elements
local layout = LinearLayout(activity)
layout.setOrientation(LinearLayout.VERTICAL)

local openDialogButton = Button(activity)
openDialogButton.setText("Add a new station")

local playStopButton = Button(activity)
playStopButton.setText("Play")

local stationTextView = TextView(activity)
stationTextView.setText("")

local displayAllStationsButton = Button(activity)
displayAllStationsButton.setText("Display All Stations")

-- Function to save station information to file
local function saveStationToFile(station)
    local file = File(STORAGE_PATH)
    if not file.exists() then
        file.getParentFile().mkdirs()
        file.createNewFile()
    end
    local fileWriter = FileWriter(file, true)
    local bufferedWriter = BufferedWriter(fileWriter)
    bufferedWriter.write("Title: " .. station.title)
    bufferedWriter.newLine()
    bufferedWriter.write("Type: " .. station.type)
    bufferedWriter.newLine()
    bufferedWriter.write("Country: " .. station.country)
    bufferedWriter.newLine()
    bufferedWriter.write("URL: " .. station.url)
    bufferedWriter.write("\n" )
    bufferedWriter.newLine()
    bufferedWriter.close()
    fileWriter.close()
end

-- Show dialog function
local function showDialog()
    local dialog = AlertDialog.Builder(activity)
    dialog.setTitle("Add a New Radio Station")
    local dialogLayout = LinearLayout(activity)
    dialogLayout.setOrientation(LinearLayout.VERTICAL)
    local titleInput = EditText(activity)
    titleInput.setHint("Title")
    dialogLayout.addView(titleInput)
    local typeInput = EditText(activity)
    typeInput.setHint("Type")
    dialogLayout.addView(typeInput)
    local countryInput = EditText(activity)
    countryInput.setHint("Country")
    dialogLayout.addView(countryInput)
    local urlInput = EditText(activity)
    urlInput.setHint("Streaming URL")
    dialogLayout.addView(urlInput)
    dialog.setView(dialogLayout)
    dialog.setPositiveButton("Save", DialogInterface.OnClickListener {
        onClick = function(_, _)
            local title = titleInput.getText().toString()
            local type = typeInput.getText().toString()
            local country = countryInput.getText().toString()
            local url = urlInput.getText().toString()
            local station = { title = title, type = type, country = country, url = url }
            saveStationToFile(station)
            displayAllStationsDialog(getAllStationInfo())
            if url ~= "" then
                local selectedUrl = getStationUrl(title)
                if selectedUrl then
                    playStation(selectedUrl)
                    stationTextView.setText(title)
                    mediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {
                        onPrepared = function(_)
                            mediaPlayer.start()
                            playStopButton.setText("Stop")
                        end
                    })
                    -- Make the ListView visible
                    -- listView.setVisibility(View.VISIBLE)  -- Uncomment and replace with your ListView reference
                else
                    Toast.makeText(activity, "URL not found for the selected station", Toast.LENGTH_SHORT).show()
                end
            end
        end
    })
    dialog.setNegativeButton("Cancel", DialogInterface.OnClickListener {
        onClick = function(_, _)
            dialog.dismiss()
        end
    })
    dialog.show()
end

openDialogButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        showDialog()
    end
})

-- Function to get all station information from file
local function getAllStationInfo()
    local file = File(STORAGE_PATH)
    local scanner = Scanner(file)
    local stations = {}  -- Change to Lua table

    while scanner.hasNextLine() do
        local line = scanner.nextLine()

        local title = line:match("Title: (.+)")
        local type = line:match("Type: (.+)")
        local country = line:match("Country: (.+)")
        local url = line:match("URL: (.+)")

        if title then
            stations[title] = {
                type = type,
                country = country,
                url = url
            }
        end
    end

    scanner.close()
    return stations
end

-- Function to get station URL from file using pattern matching
local function getStationUrl(title)
    local file = File(STORAGE_PATH)
    local scanner = Scanner(file)
    local url
    local pattern = "Title: " .. title .. "\nType: .-\nCountry: .-\nURL: (.-)\n"
    local content = scanner.useDelimiter("\\Z").next()
    url = content:match(pattern)
    scanner.close()
    -- Check if the URL starts with "http" before returning
    if url and url:find("^http") then
        return url
    else
        return nil
    end
end

-- MediaPlayer setup function
local mediaPlayer = MediaPlayer()
mediaPlayer.setAudioStreamType(AudioManager.STREAM_MUSIC)

-- Function to play a specific station
local function playStation(url)
    mediaPlayer.reset()
    mediaPlayer.setDataSource(url)
    mediaPlayer.prepareAsync()
end

playStopButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        if mediaPlayer.isPlaying() then
            mediaPlayer.pause()
            playStopButton.setText("Play")
        else
            mediaPlayer.start()
            playStopButton.setText("Stop")
        end
    end
})

-- Function to display all stations in a ListView
local function displayAllStationsDialog(stations)
    local dialog = AlertDialog.Builder(activity)
    dialog.setTitle("Stations List")

    local localListView = ListView(activity)  -- Create a new instance of ListView

    local stationTitles = {}  -- Store titles for easy access
    for title, _ in pairs(stations) do
        table.insert(stationTitles, title)
    end

    local adapter = ArrayAdapter(activity, android.R.layout.simple_list_item_1, stationTitles)
    localListView.setAdapter(adapter)

    dialog.setView(localListView)

    local alertDialog = dialog.create()

-- Function to remove station information from the original file with confirmation dialog
local function removeStationWithConfirmation(title)
    local confirmationDialog = AlertDialog.Builder(activity)
    confirmationDialog.setTitle("Confirmation")
    confirmationDialog.setMessage("Are you sure you want to remove " .. title .. "?")

    confirmationDialog.setPositiveButton("Yes", DialogInterface.OnClickListener {
        onClick = function(_, _)
            local file = File(STORAGE_PATH)
            local tempFile = File("/storage/emulated/0/解说/Tools/radio-player/temp_data.txt")
            local scanner = Scanner(file)
            local writer = BufferedWriter(FileWriter(tempFile))
            local pattern = "Title: " .. title .. "\nType: .-\nCountry: .-\nURL: .-\n"
            local content = scanner.useDelimiter("\\Z").next()
            local filteredContent = content:gsub(pattern, "")

            writer.write(filteredContent)
            scanner.close()
            writer.close()

            -- Delete the original file
            file.delete()

            -- Rename the temporary file to the original file name
            tempFile.renameTo(file)

            -- Refresh the station list in the dialog
            displayAllStationsDialog(getAllStationInfo())

            Toast.makeText(activity, "Removed: " .. title, Toast.LENGTH_SHORT).show()
        end
    })

    confirmationDialog.setNegativeButton("No", DialogInterface.OnClickListener {
        onClick = function(_, _)
            -- Do nothing if "No" is clicked
        end
    })

    confirmationDialog.show()
end

    localListView.setOnItemLongClickListener(AdapterView.OnItemLongClickListener {
        onItemLongClick = function(_, _, position, _)
            local selectedStationTitle = stationTitles[position + 1]  -- Lua tables are 1-indexed
            local popupMenu = PopupMenu(activity, localListView.getChildAt(position))

            -- Use direct addition of menu items
           -- popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 1, "Add to Favorites")
            popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 2, "Remove")
            --popupMenu.getMenu().add(Menu.NONE, Menu.NONE, 3, "Share")

            popupMenu.setOnMenuItemClickListener(PopupMenu.OnMenuItemClickListener {
                onMenuItemClick = function(item)
                    if item.getTitle() == "Add to Favorites" then
                        -- Implement your logic for adding to favorites
                        Toast.makeText(activity, "Added to Favorites: " .. selectedStationTitle, Toast.LENGTH_SHORT).show()
                    elseif item.getTitle() == "Remove" then
                        -- Implement your logic for removing the station
                        removeStationWithConfirmation(selectedStationTitle)
                    elseif item.getTitle() == "Share" then
                        -- Implement your logic for sharing the station
                        Toast.makeText(activity, "Shared: " .. selectedStationTitle, Toast.LENGTH_SHORT).show()
                    end
                    return true
                end
            })
            
            popupMenu.show()
            return true
        end
    })

    localListView.setOnItemClickListener(AdapterView.OnItemClickListener {
        onItemClick = function(_, _, position, _)
            local selectedStationTitle = stationTitles[position +1]  -- Lua tables are 1-indexed
            local selectedUrl = getStationUrl(selectedStationTitle) 

            if selectedUrl then
                playStation(selectedUrl)
                stationTextView.setText(selectedStationTitle)
                mediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {
                    onPrepared = function(_)
                        mediaPlayer.start()
                        playStopButton.setText("Stop")
                    end
                })
                -- Make the ListView visible
                -- localListView.setVisibility(View.VISIBLE)  -- Uncomment and replace with your ListView reference
            else
                Toast.makeText(activity, "URL not found for the selected station", Toast.LENGTH_SHORT).show()
            end
        end
    })

    alertDialog.show()
end

displayAllStationsButton.setOnClickListener(View.OnClickListener {
    onClick = function(_)
        displayAllStationsDialog(getAllStationInfo())
    end
})

-- Function to get the current playing station title
local function getCurrentStationTitle()
    -- Implement logic to get the current station title
    return "BBC News"  -- Replace with your logic
end

-- Function to get the current playing station URL
local function getCurrentStationUrl()
    -- Implement logic to get the current station URL
    return "http://example.com/stream"  -- Replace with your logic
end

-- Function to create and show a media-style notification
local function showNotification(title, content)
    -- Implementation of showNotification
    -- ...
end

-- Function to pause the station playback
local function pauseStation()
    if mediaPlayer.isPlaying() then
        mediaPlayer.pause()

        -- Update the notification for the paused station
        showNotification("Paused", getCurrentStationTitle())
    end
end

-- Broadcast receiver for handling media actions from the notification
local mediaActionReceiver = BroadcastReceiver {
    onReceive = function(context, intent)
        local action = intent.getAction()

        if action == "PLAY_ACTION" then
            playStation(getCurrentStationUrl(), getCurrentStationTitle())
        elseif action == "PAUSE_ACTION" then
            pauseStation()
        end
    end
}

-- Broadcast receiver for handling media actions from the notification
local mediaActionReceiver = BroadcastReceiver {
    onReceive = function(context, intent)
        local action = intent.getAction()

        if action == "PLAY_ACTION" then
            playStation(getCurrentStationUrl(), getCurrentStationTitle())
        elseif action == "PAUSE_ACTION" then
            pauseStation()
        end
    end
}

-- Function to create and show a media-style notification
local function showNotification(title, content)
    local notificationManager = activity.getSystemService(Context.NOTIFICATION_SERVICE)
    local channelId = "RadioPlayerChannel"

    -- Create a NotificationChannel (required for Android 8.0 and above)
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) then
        local channel = NotificationChannel(channelId, "Radio Player Channel", NotificationManager.IMPORTANCE_DEFAULT)
        notificationManager.createNotificationChannel(channel)
    end

    -- Register the media action receiver
    activity.registerReceiver(mediaActionReceiver, IntentFilter("PLAY_ACTION"))
    activity.registerReceiver(mediaActionReceiver, IntentFilter("PAUSE_ACTION"))

    -- Create media-style notification
    local mediaStyle = Notification.MediaStyle()
        .setMediaSession(nil)  -- Set to nil for simplicity

-- Update the playActionIntent and pauseActionIntent creation in the showNotification function
local playActionIntent = Intent("PLAY_ACTION")
local playActionPendingIntent = PendingIntent.getBroadcast(activity, 0, playActionIntent, PendingIntent.FLAG_IMMUTABLE)

local pauseActionIntent = Intent("PAUSE_ACTION")
local pauseActionPendingIntent = PendingIntent.getBroadcast(activity, 0, pauseActionIntent, PendingIntent.FLAG_IMMUTABLE)

    local notificationBuilder = Notification.Builder(activity, channelId)
        .setSmallIcon(android.R.drawable.ic_media_play)  -- Change the icon as needed
        .setContentTitle(title)
        .setContentText(content)
        .setStyle(mediaStyle)
        .setShowWhen(false)  -- Hide timestamp
        .setOngoing(true)  -- Make the notification ongoing (won't be dismissed by user swipe)
        .setContentIntent(playActionPendingIntent)  -- Set click action to play action
        .setPriority(Notification.PRIORITY_DEFAULT)
        .setAutoCancel(true)  -- Close the notification when clicked
        .addAction(android.R.drawable.ic_media_play, "Play", playActionPendingIntent)
        .addAction(android.R.drawable.ic_media_pause, "Pause", pauseActionPendingIntent)

    -- Show the notification
    notificationManager.notify(1, notificationBuilder.build())
end

-- Update the playStation and pauseStation functions
local function playStation(url, title)
    mediaPlayer.reset()
    mediaPlayer.setDataSource(url)
    mediaPlayer.prepareAsync()

    -- Update the notification for the playing station
    showNotification("Now Playing", title)
end

local function pauseStation()
    if mediaPlayer.isPlaying() then
        mediaPlayer.pause()

        -- Update the notification for the paused station
        showNotification("Paused", getCurrentStationTitle())
    end
end

-- Function to get the current playing station title
local function getCurrentStationTitle()
    -- Implement logic to get the current station title
    return "BBC News"  -- Replace with your logic
end

-- Function to get the current playing station URL
local function getCurrentStationUrl()
    -- Implement logic to get the current station URL
    return "http://example.com/stream"  -- Replace with your logic
end

showNotification(title, content)

layout.addView(openDialogButton)
layout.addView(playStopButton)
layout.addView(stationTextView)
layout.addView(displayAllStationsButton)

activity.setContentView(layout)